/*
 Copyright 2009 Virginia Polytechnic Institute and State University  

 Licensed under the Apache License, Version 2.0 (the "License"); 
 you may not use this file except in compliance with the License. 
 You may obtain a copy of the License at 
 
 http://www.apache.org/licenses/LICENSE-2.0 

 Unless required by applicable law or agreed to in writing, software 
 distributed under the License is distributed on an "AS IS" BASIS, 
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 See the License for the specific language governing permissions and 
 limitations under the License. 
*/

/*! Implementation file for the VCROSS Cognitive Radio public API defined in
 * include/libvtcross.h.
 *
 * libvtcross provides the interface that radio host platforms and applications
 * use to talk to the CROSS radio system.  All external communication with the
 * CROSS radio occurs through this library.
 */

#include <cstdlib>
#include <string>

#include "vtcross/common.h"
#include "vtcross/debug.h"
#include "vtcross/libvtcross.h"

using namespace std;


/* Strings that store the remote (or local) shell location for use within the
 * ConnectToShell() function.  Note that these values must be set with the
 * SetCrossShellLocation public function within the client code. */
string shellHostname;
string shellPort;


uint32_t
ConnectToShell()
{
    return ClientSocket(shellHostname.c_str(), shellPort.c_str());
}


void
SetCrossShellLocation(string hostname, string port)
{
    shellHostname = hostname;
    shellPort = port;
}


// TODO the following three functions all do exactly the same thing.  Why not
// simply combine them?

Parameter* 
GetOptimalParameters(Observable *radioObservables, uint32_t numObservables, Parameter *currentParameters, uint32_t numCurrentParameters, Utility *radioUtilities, uint32_t numUtilities)
{
    char var[50];
    char counter[55];
    char buffer[256];

    uint32_t socketFD = ConnectToShell();
    SendMessage(socketFD, "request_optimization");

    /* Get number of observables to send.  This information needs to be 
     * sent to the Cognitive Radio Shell also.   
     */

    /* Send Observables */
    sprintf(counter, "%i", numObservables);
    SendMessage(socketFD, counter);
    for(size_t i = 0; i < numObservables; i++) {
        SendMessage(socketFD, radioObservables[i].name.c_str());
        sprintf(var, "%f", radioObservables[i].value);
        SendMessage(socketFD, var);    
    }

    /* Send Parameters */
    memset(counter, 0, 55);
    sprintf(counter, "%i", numCurrentParameters);
    SendMessage(socketFD, counter);
    for(size_t i = 0; i < numCurrentParameters; i++) {
        SendMessage(socketFD,currentParameters[i].name.c_str());
        sprintf(var,"%f",currentParameters[i].value);
        SendMessage(socketFD,var);    
    }

    /* Send Utilities */
    memset(counter, 0, 55);
    sprintf(counter, "%i", numUtilities);
    SendMessage(socketFD, counter);
    for(size_t i = 0; i < numUtilities; i++) {
        SendMessage(socketFD,radioUtilities[i].name.c_str());
        sprintf(var,"%f",radioUtilities[i].value);
        SendMessage(socketFD,var);
    }
	

    /* Receive Set of Parameters */
    memset(buffer, 0, 256);
    ReadMessage(socketFD, buffer);
    uint32_t numParameters = atoi(buffer);
    Parameter *pList = new Parameter[numParameters];
    
    for(ssize_t i = 0; i < numParameters; i++) {
        memset(buffer, 0, 256);
        ReadMessage(socketFD, buffer);
        pList[i].name = std::string(buffer);
        
        memset(buffer, 0, 256);
        ReadMessage(socketFD, buffer);
        pList[i].value = atof(buffer);
    }   

    close(socketFD);
    return pList;
}

bool
UpdateParameterPerformance(Parameter *p, uint32_t numParameters, Observable *o, uint32_t numObservables, Utility *u, uint32_t numUtilities)
{
    char counter[55];
    char var[50];

    uint32_t socketFD = ConnectToShell();
    SendMessage(socketFD, "update_performance");
    
    // Send Parameters 
    memset(counter, 0, 55);
    sprintf(counter, "%i", numParameters);
    SendMessage(socketFD, counter);
    
    for(size_t i = 0; i < numParameters; i++) {
        SendMessage(socketFD, p[i].name.c_str());
        sprintf(var, "%f", p[i].value);
        SendMessage(socketFD, var);    
    }
    
    // Send Observables
    memset(counter, 0, 55);
    sprintf(counter, "%i", numObservables);
    SendMessage(socketFD, counter);
    for(size_t i = 0; i < numObservables; i++) {
        SendMessage(socketFD, o[i].name.c_str());
        sprintf(var, "%f", o[i].value);
        SendMessage(socketFD, var);    
    }

    // Send Utilities
    memset(counter, 0, 55);
    sprintf(counter, "%i", numUtilities);
    SendMessage(socketFD, counter);
    for(size_t i = 0; i < numUtilities; i++) {
	SendMessage(socketFD, u[i].name.c_str());
	sprintf(var, "%f", u[i].value);
	SendMessage(socketFD, var);
    }
	
}

uint32_t GetNum(string type)
{

	char buffer[256];
	uint32_t socketfd = ConnectToShell();
	
	if (type == "parameters")
		SendMessage(socketfd, "get_number_parameters");
	else if (type == "utilities")
		SendMessage(socketfd, "get_number_utilities");
	else if (type == "observables")
		SendMessage(socketfd, "get_number_observables");	
	else
		LOG("Type not included, either 'parameters' or 'utilities' or 'observables'");	

	memset(buffer, 0, 256);
	ReadMessage(socketfd, buffer);
	uint32_t number = atoi(buffer);
	return number;


}
